/* We don't want gtkdoc scanning this file, it'll get
 * confused by seeing function definitions, so mark it as: */
/*<private_header>*/

/**
 * emp_cli_channel_handler_callback_for_handle_channel:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a HandleChannel method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_handler_collect_callback_handle_channel (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_channel_handler_invoke_callback_handle_channel (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_handler_callback_for_handle_channel callback = (emp_cli_channel_handler_callback_for_handle_channel) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_handler_call_handle_channel:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel_Type: Used to pass an 'in' argument: The channel type
 * @in_Channel: Used to pass an 'in' argument: The object-path of the channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a HandleChannel method call.
 *
 * Called when a channel handler should handle a new channel.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_handler_call_handle_channel (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel_Type,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    emp_cli_channel_handler_callback_for_handle_channel callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_HANDLER;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "HandleChannel",
          G_TYPE_STRING, in_Bus_Name,
          DBUS_TYPE_G_OBJECT_PATH, in_Connection,
          G_TYPE_STRING, in_Channel_Type,
          DBUS_TYPE_G_OBJECT_PATH, in_Channel,
          G_TYPE_UINT, in_Handle_Type,
          G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "HandleChannel", iface,
          _emp_cli_channel_handler_invoke_callback_handle_channel,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "HandleChannel",
              _emp_cli_channel_handler_collect_callback_handle_channel,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              G_TYPE_STRING, in_Channel_Type,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_channel_handler_run_state_handle_channel;
static void
_emp_cli_channel_handler_finish_running_handle_channel (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_handler_run_state_handle_channel *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_handler_run_handle_channel:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel_Type: Used to pass an 'in' argument: The channel type
 * @in_Channel: Used to pass an 'in' argument: The object-path of the channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method HandleChannel and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Called when a channel handler should handle a new channel.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_handler_run_handle_channel (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel_Type,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_HANDLER;
  TpProxyPendingCall *pc;
  _emp_cli_channel_handler_run_state_handle_channel state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "HandleChannel", iface,
      _emp_cli_channel_handler_finish_running_handle_channel,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "HandleChannel",
          _emp_cli_channel_handler_collect_callback_handle_channel,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              G_TYPE_STRING, in_Channel_Type,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


static inline void
emp_cli_add_signals_for_channel_interface_tube (DBusGProxy *proxy)
{
  if (!tp_proxy_dbus_g_proxy_claim_for_signal_adding (proxy))
    return;
  dbus_g_proxy_add_signal (proxy, "TubeChannelStateChanged",
      G_TYPE_UINT,
      G_TYPE_INVALID);
}


/**
 * emp_cli_channel_interface_tube_signal_callback_tube_channel_state_changed:
 * @proxy: The proxy on which emp_cli_channel_interface_tube_connect_to_tube_channel_state_changed ()
 *  was called
 * @arg_state: The new state of the tube; see the Tube_Channel_State enumeration.
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal TubeChannelStateChanged.
 */
static void
_emp_cli_channel_interface_tube_collect_args_of_tube_channel_state_changed (DBusGProxy *proxy G_GNUC_UNUSED,
    guint arg_state,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (1);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, G_TYPE_UINT);
  g_value_set_uint (args->values + 0, arg_state);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_emp_cli_channel_interface_tube_invoke_callback_for_tube_channel_state_changed (TpProxy *tpproxy,
    GError *error G_GNUC_UNUSED,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_interface_tube_signal_callback_tube_channel_state_changed callback =
      (emp_cli_channel_interface_tube_signal_callback_tube_channel_state_changed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_uint (args->values + 0),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * emp_cli_channel_interface_tube_connect_to_tube_channel_state_changed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal TubeChannelStateChanged.
 *
 * Emitted when the state of the tube channel changes.
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
emp_cli_channel_interface_tube_connect_to_tube_channel_state_changed (TpProxy *proxy,
    emp_cli_channel_interface_tube_signal_callback_tube_channel_state_changed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[2] = {
      G_TYPE_UINT,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      EMP_IFACE_QUARK_CHANNEL_INTERFACE_TUBE, "TubeChannelStateChanged",
      expected_types,
      G_CALLBACK (_emp_cli_channel_interface_tube_collect_args_of_tube_channel_state_changed),
      _emp_cli_channel_interface_tube_invoke_callback_for_tube_channel_state_changed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

static inline void
emp_cli_add_signals_for_channel_type_dbus_tube (DBusGProxy *proxy)
{
  if (!tp_proxy_dbus_g_proxy_claim_for_signal_adding (proxy))
    return;
  dbus_g_proxy_add_signal (proxy, "DBusNamesChanged",
      (dbus_g_type_get_map ("GHashTable", G_TYPE_UINT, G_TYPE_STRING)),
      DBUS_TYPE_G_UINT_ARRAY,
      G_TYPE_INVALID);
}


/**
 * emp_cli_channel_type_dbus_tube_signal_callback_d_bus_names_changed:
 * @proxy: The proxy on which emp_cli_channel_type_dbus_tube_connect_to_d_bus_names_changed ()
 *  was called
 * @arg_added: Array of handles and D-Bus names of new participants.
 * @arg_removed: Array of handles of former participants.
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal DBusNamesChanged.
 */
static void
_emp_cli_channel_type_dbus_tube_collect_args_of_d_bus_names_changed (DBusGProxy *proxy G_GNUC_UNUSED,
    GHashTable *arg_added,
    const GArray *arg_removed,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (2);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 2; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, (dbus_g_type_get_map ("GHashTable", G_TYPE_UINT, G_TYPE_STRING)));
  g_value_set_boxed (args->values + 0, arg_added);

  g_value_unset (args->values + 1);
  g_value_init (args->values + 1, DBUS_TYPE_G_UINT_ARRAY);
  g_value_set_boxed (args->values + 1, arg_removed);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_emp_cli_channel_type_dbus_tube_invoke_callback_for_d_bus_names_changed (TpProxy *tpproxy,
    GError *error G_GNUC_UNUSED,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_dbus_tube_signal_callback_d_bus_names_changed callback =
      (emp_cli_channel_type_dbus_tube_signal_callback_d_bus_names_changed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      g_value_get_boxed (args->values + 1),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * emp_cli_channel_type_dbus_tube_connect_to_d_bus_names_changed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal DBusNamesChanged.
 *
 * Emitted on a multi-user (i.e. Handle_Type_Room) D-Bus tube when a         participant opens or closes the tube.  This provides change         notification for the &lt;tp:member-ref&gt;DBusNames&lt;/tp:member-ref&gt; property.
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
emp_cli_channel_type_dbus_tube_connect_to_d_bus_names_changed (TpProxy *proxy,
    emp_cli_channel_type_dbus_tube_signal_callback_d_bus_names_changed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[3] = {
      (dbus_g_type_get_map ("GHashTable", G_TYPE_UINT, G_TYPE_STRING)),
      DBUS_TYPE_G_UINT_ARRAY,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE, "DBusNamesChanged",
      expected_types,
      G_CALLBACK (_emp_cli_channel_type_dbus_tube_collect_args_of_d_bus_names_changed),
      _emp_cli_channel_type_dbus_tube_invoke_callback_for_d_bus_names_changed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * emp_cli_channel_type_dbus_tube_callback_for_offer_d_bus_tube:
 * @proxy: the proxy on which the call was made
 * @out_address: Used to return an 'out' argument if @error is %NULL: The string describing the address of the private bus. The client           SHOULD not attempt to connect to the address until the tube is open.
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a OfferDBusTube method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_type_dbus_tube_collect_callback_offer_d_bus_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;
  GValueArray *args;
  GValue blank = { 0 };
  guint i;
  gchar *out_address;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_STRING, &out_address,
      G_TYPE_INVALID);

  if (error != NULL)
    {
      tp_proxy_pending_call_v0_take_results (user_data, error,
          NULL);
      return;
    }

  args = g_value_array_new (1);
  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, G_TYPE_STRING);
  g_value_take_string (args->values + 0, out_address);
  tp_proxy_pending_call_v0_take_results (user_data, NULL, args);
}
static void
_emp_cli_channel_type_dbus_tube_invoke_callback_offer_d_bus_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_dbus_tube_callback_for_offer_d_bus_tube callback = (emp_cli_channel_type_dbus_tube_callback_for_offer_d_bus_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          NULL,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      g_value_get_string (args->values + 0),
      error, user_data, weak_object);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_dbus_tube_call_offer_d_bus_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_parameters: Used to pass an 'in' argument: The dictionary of arbitrary           &lt;tp:dbus-ref namespace=&quot;org.freedesktop.Telepathy.Channel.Interface.Tube.DRAFT&quot;&gt;Parameters&lt;/tp:dbus-ref&gt;           to send with the tube offer.
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a OfferDBusTube method call.
 *
 * Offers a D-Bus tube providing the service specified.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_type_dbus_tube_call_offer_d_bus_tube (TpProxy *proxy,
    gint timeout_ms,
    GHashTable *in_parameters,
    emp_cli_channel_type_dbus_tube_callback_for_offer_d_bus_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            NULL,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "OfferDBusTube",
          (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "OfferDBusTube", iface,
          _emp_cli_channel_type_dbus_tube_invoke_callback_offer_d_bus_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "OfferDBusTube",
              _emp_cli_channel_type_dbus_tube_collect_callback_offer_d_bus_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gchar **out_address;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_channel_type_dbus_tube_run_state_offer_d_bus_tube;
static void
_emp_cli_channel_type_dbus_tube_finish_running_offer_d_bus_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_type_dbus_tube_run_state_offer_d_bus_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (state->out_address != NULL)
    *state->out_address = g_value_dup_string (args->values + 0);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_dbus_tube_run_offer_d_bus_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_parameters: Used to pass an 'in' argument: The dictionary of arbitrary           &lt;tp:dbus-ref namespace=&quot;org.freedesktop.Telepathy.Channel.Interface.Tube.DRAFT&quot;&gt;Parameters&lt;/tp:dbus-ref&gt;           to send with the tube offer.
 * @out_address: Used to return an 'out' argument if %TRUE is returned: The string describing the address of the private bus. The client           SHOULD not attempt to connect to the address until the tube is open.
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method OfferDBusTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Offers a D-Bus tube providing the service specified.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_type_dbus_tube_run_offer_d_bus_tube (TpProxy *proxy,
    gint timeout_ms,
    GHashTable *in_parameters,
    gchar **out_address,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE;
  TpProxyPendingCall *pc;
  _emp_cli_channel_type_dbus_tube_run_state_offer_d_bus_tube state = {
      NULL /* loop */, error,
    out_address,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "OfferDBusTube", iface,
      _emp_cli_channel_type_dbus_tube_finish_running_offer_d_bus_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "OfferDBusTube",
          _emp_cli_channel_type_dbus_tube_collect_callback_offer_d_bus_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_channel_type_dbus_tube_callback_for_accept_d_bus_tube:
 * @proxy: the proxy on which the call was made
 * @out_address: Used to return an 'out' argument if @error is %NULL: The string describing the address of the private bus. The client           SHOULD not attempt to connect to the address until the tube is open.
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a AcceptDBusTube method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_type_dbus_tube_collect_callback_accept_d_bus_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;
  GValueArray *args;
  GValue blank = { 0 };
  guint i;
  gchar *out_address;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_STRING, &out_address,
      G_TYPE_INVALID);

  if (error != NULL)
    {
      tp_proxy_pending_call_v0_take_results (user_data, error,
          NULL);
      return;
    }

  args = g_value_array_new (1);
  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, G_TYPE_STRING);
  g_value_take_string (args->values + 0, out_address);
  tp_proxy_pending_call_v0_take_results (user_data, NULL, args);
}
static void
_emp_cli_channel_type_dbus_tube_invoke_callback_accept_d_bus_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_dbus_tube_callback_for_accept_d_bus_tube callback = (emp_cli_channel_type_dbus_tube_callback_for_accept_d_bus_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          NULL,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      g_value_get_string (args->values + 0),
      error, user_data, weak_object);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_dbus_tube_call_accept_d_bus_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a AcceptDBusTube method call.
 *
 * Accept a D-Bus tube that&apos;s in the &amp;quot;local pending&amp;quot; state. The         connection manager will attempt to open the tube. The tube remains in         the &amp;quot;local pending&amp;quot; state until the TubeStateChanged signal is         emitted.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_type_dbus_tube_call_accept_d_bus_tube (TpProxy *proxy,
    gint timeout_ms,
    emp_cli_channel_type_dbus_tube_callback_for_accept_d_bus_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            NULL,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "AcceptDBusTube",
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "AcceptDBusTube", iface,
          _emp_cli_channel_type_dbus_tube_invoke_callback_accept_d_bus_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "AcceptDBusTube",
              _emp_cli_channel_type_dbus_tube_collect_callback_accept_d_bus_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gchar **out_address;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_channel_type_dbus_tube_run_state_accept_d_bus_tube;
static void
_emp_cli_channel_type_dbus_tube_finish_running_accept_d_bus_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_type_dbus_tube_run_state_accept_d_bus_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (state->out_address != NULL)
    *state->out_address = g_value_dup_string (args->values + 0);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_dbus_tube_run_accept_d_bus_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @out_address: Used to return an 'out' argument if %TRUE is returned: The string describing the address of the private bus. The client           SHOULD not attempt to connect to the address until the tube is open.
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method AcceptDBusTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Accept a D-Bus tube that&apos;s in the &amp;quot;local pending&amp;quot; state. The         connection manager will attempt to open the tube. The tube remains in         the &amp;quot;local pending&amp;quot; state until the TubeStateChanged signal is         emitted.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_type_dbus_tube_run_accept_d_bus_tube (TpProxy *proxy,
    gint timeout_ms,
    gchar **out_address,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE;
  TpProxyPendingCall *pc;
  _emp_cli_channel_type_dbus_tube_run_state_accept_d_bus_tube state = {
      NULL /* loop */, error,
    out_address,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "AcceptDBusTube", iface,
      _emp_cli_channel_type_dbus_tube_finish_running_accept_d_bus_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "AcceptDBusTube",
          _emp_cli_channel_type_dbus_tube_collect_callback_accept_d_bus_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


static inline void
emp_cli_add_signals_for_channel_type_stream_tube (DBusGProxy *proxy)
{
  if (!tp_proxy_dbus_g_proxy_claim_for_signal_adding (proxy))
    return;
  dbus_g_proxy_add_signal (proxy, "StreamTubeNewConnection",
      G_TYPE_UINT,
      G_TYPE_INVALID);
}


/**
 * emp_cli_channel_type_stream_tube_signal_callback_stream_tube_new_connection:
 * @proxy: The proxy on which emp_cli_channel_type_stream_tube_connect_to_stream_tube_new_connection ()
 *  was called
 * @arg_handle: The handle of the participant who opened the new connection
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal StreamTubeNewConnection.
 */
static void
_emp_cli_channel_type_stream_tube_collect_args_of_stream_tube_new_connection (DBusGProxy *proxy G_GNUC_UNUSED,
    guint arg_handle,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (1);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, G_TYPE_UINT);
  g_value_set_uint (args->values + 0, arg_handle);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_emp_cli_channel_type_stream_tube_invoke_callback_for_stream_tube_new_connection (TpProxy *tpproxy,
    GError *error G_GNUC_UNUSED,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_stream_tube_signal_callback_stream_tube_new_connection callback =
      (emp_cli_channel_type_stream_tube_signal_callback_stream_tube_new_connection) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_uint (args->values + 0),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * emp_cli_channel_type_stream_tube_connect_to_stream_tube_new_connection:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal StreamTubeNewConnection.
 *
 * Emitted on a stream tube when a participant opens a new connection         to its socket.
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
emp_cli_channel_type_stream_tube_connect_to_stream_tube_new_connection (TpProxy *proxy,
    emp_cli_channel_type_stream_tube_signal_callback_stream_tube_new_connection callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[2] = {
      G_TYPE_UINT,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE, "StreamTubeNewConnection",
      expected_types,
      G_CALLBACK (_emp_cli_channel_type_stream_tube_collect_args_of_stream_tube_new_connection),
      _emp_cli_channel_type_stream_tube_invoke_callback_for_stream_tube_new_connection,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * emp_cli_channel_type_stream_tube_callback_for_offer_stream_tube:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a OfferStreamTube method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_type_stream_tube_collect_callback_offer_stream_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_channel_type_stream_tube_invoke_callback_offer_stream_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_stream_tube_callback_for_offer_stream_tube callback = (emp_cli_channel_type_stream_tube_callback_for_offer_stream_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_type_stream_tube_call_offer_stream_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_address_type: Used to pass an 'in' argument: The type of the listening address of the local service, as a member of           Socket_Address_Type.
 * @in_address: Used to pass an 'in' argument: The listening address of the local service, as indicated by the           address_type.
 * @in_access_control: Used to pass an 'in' argument: The access control the local service applies to the local socket,           specified so the connection manager can behave appropriately           when it connects.
 * @in_access_control_param: Used to pass an 'in' argument: A parameter for the access control type, to be interpreted as           specified in the documentation for the Socket_Access_Control enum.
 * @in_parameters: Used to pass an 'in' argument: The dictionary of arbitrary           &lt;tp:dbus-ref namespace=&quot;org.freedesktop.Telepathy.Channel.Interface.Tube.DRAFT&quot;&gt;Parameters&lt;/tp:dbus-ref&gt;           to send with the tube offer.
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a OfferStreamTube method call.
 *
 * Offer a stream tube exporting the local socket specified.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_type_stream_tube_call_offer_stream_tube (TpProxy *proxy,
    gint timeout_ms,
    guint in_address_type,
    const GValue *in_address,
    guint in_access_control,
    const GValue *in_access_control_param,
    GHashTable *in_parameters,
    emp_cli_channel_type_stream_tube_callback_for_offer_stream_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "OfferStreamTube",
          G_TYPE_UINT, in_address_type,
          G_TYPE_VALUE, in_address,
          G_TYPE_UINT, in_access_control,
          G_TYPE_VALUE, in_access_control_param,
          (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "OfferStreamTube", iface,
          _emp_cli_channel_type_stream_tube_invoke_callback_offer_stream_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "OfferStreamTube",
              _emp_cli_channel_type_stream_tube_collect_callback_offer_stream_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_UINT, in_address_type,
              G_TYPE_VALUE, in_address,
              G_TYPE_UINT, in_access_control,
              G_TYPE_VALUE, in_access_control_param,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_channel_type_stream_tube_run_state_offer_stream_tube;
static void
_emp_cli_channel_type_stream_tube_finish_running_offer_stream_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_type_stream_tube_run_state_offer_stream_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_channel_type_stream_tube_run_offer_stream_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_address_type: Used to pass an 'in' argument: The type of the listening address of the local service, as a member of           Socket_Address_Type.
 * @in_address: Used to pass an 'in' argument: The listening address of the local service, as indicated by the           address_type.
 * @in_access_control: Used to pass an 'in' argument: The access control the local service applies to the local socket,           specified so the connection manager can behave appropriately           when it connects.
 * @in_access_control_param: Used to pass an 'in' argument: A parameter for the access control type, to be interpreted as           specified in the documentation for the Socket_Access_Control enum.
 * @in_parameters: Used to pass an 'in' argument: The dictionary of arbitrary           &lt;tp:dbus-ref namespace=&quot;org.freedesktop.Telepathy.Channel.Interface.Tube.DRAFT&quot;&gt;Parameters&lt;/tp:dbus-ref&gt;           to send with the tube offer.
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method OfferStreamTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Offer a stream tube exporting the local socket specified.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_type_stream_tube_run_offer_stream_tube (TpProxy *proxy,
    gint timeout_ms,
    guint in_address_type,
    const GValue *in_address,
    guint in_access_control,
    const GValue *in_access_control_param,
    GHashTable *in_parameters,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE;
  TpProxyPendingCall *pc;
  _emp_cli_channel_type_stream_tube_run_state_offer_stream_tube state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "OfferStreamTube", iface,
      _emp_cli_channel_type_stream_tube_finish_running_offer_stream_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "OfferStreamTube",
          _emp_cli_channel_type_stream_tube_collect_callback_offer_stream_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_UINT, in_address_type,
              G_TYPE_VALUE, in_address,
              G_TYPE_UINT, in_access_control,
              G_TYPE_VALUE, in_access_control_param,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_parameters,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_channel_type_stream_tube_callback_for_accept_stream_tube:
 * @proxy: the proxy on which the call was made
 * @out_address: Used to return an 'out' argument if @error is %NULL: The address on which the connection manager will listen for           connections to this tube. The client should not attempt to connect           to the address until the tube is open.
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a AcceptStreamTube method call
 * succeeds or fails.
 */
static void
_emp_cli_channel_type_stream_tube_collect_callback_accept_stream_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;
  GValueArray *args;
  GValue blank = { 0 };
  guint i;
  GValue *out_address = g_new0 (GValue, 1);

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_VALUE, out_address,
      G_TYPE_INVALID);

  if (error != NULL)
    {
      tp_proxy_pending_call_v0_take_results (user_data, error,
          NULL);
      g_free (out_address);
      return;
    }

  args = g_value_array_new (1);
  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, G_TYPE_VALUE);
  g_value_take_boxed (args->values + 0, out_address);
  tp_proxy_pending_call_v0_take_results (user_data, NULL, args);
}
static void
_emp_cli_channel_type_stream_tube_invoke_callback_accept_stream_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_channel_type_stream_tube_callback_for_accept_stream_tube callback = (emp_cli_channel_type_stream_tube_callback_for_accept_stream_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          NULL,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      g_value_get_boxed (args->values + 0),
      error, user_data, weak_object);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_stream_tube_call_accept_stream_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_address_type: Used to pass an 'in' argument: The type of address the connection manager should listen on.
 * @in_access_control: Used to pass an 'in' argument: The type of access control the connection manager should apply to           the socket.
 * @in_access_control_param: Used to pass an 'in' argument: A parameter for the access control type, to be interpreted as           specified in the documentation for the Socket_Access_Control enum.
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a AcceptStreamTube method call.
 *
 * Accept a stream tube that&apos;s in the &amp;quot;local pending&amp;quot; state. The         connection manager will attempt to open the tube. The tube remains in         the &amp;quot;local pending&amp;quot; state until the TubeStateChanged signal is         emitted.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_channel_type_stream_tube_call_accept_stream_tube (TpProxy *proxy,
    gint timeout_ms,
    guint in_address_type,
    guint in_access_control,
    const GValue *in_access_control_param,
    emp_cli_channel_type_stream_tube_callback_for_accept_stream_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            NULL,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "AcceptStreamTube",
          G_TYPE_UINT, in_address_type,
          G_TYPE_UINT, in_access_control,
          G_TYPE_VALUE, in_access_control_param,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "AcceptStreamTube", iface,
          _emp_cli_channel_type_stream_tube_invoke_callback_accept_stream_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "AcceptStreamTube",
              _emp_cli_channel_type_stream_tube_collect_callback_accept_stream_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_UINT, in_address_type,
              G_TYPE_UINT, in_access_control,
              G_TYPE_VALUE, in_access_control_param,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    GValue **out_address;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_channel_type_stream_tube_run_state_accept_stream_tube;
static void
_emp_cli_channel_type_stream_tube_finish_running_accept_stream_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_channel_type_stream_tube_run_state_accept_stream_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (state->out_address != NULL)
    *state->out_address = g_value_dup_boxed (args->values + 0);

  g_value_array_free (args);
}

/**
 * emp_cli_channel_type_stream_tube_run_accept_stream_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_address_type: Used to pass an 'in' argument: The type of address the connection manager should listen on.
 * @in_access_control: Used to pass an 'in' argument: The type of access control the connection manager should apply to           the socket.
 * @in_access_control_param: Used to pass an 'in' argument: A parameter for the access control type, to be interpreted as           specified in the documentation for the Socket_Access_Control enum.
 * @out_address: Used to return an 'out' argument if %TRUE is returned: The address on which the connection manager will listen for           connections to this tube. The client should not attempt to connect           to the address until the tube is open.
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method AcceptStreamTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Accept a stream tube that&apos;s in the &amp;quot;local pending&amp;quot; state. The         connection manager will attempt to open the tube. The tube remains in         the &amp;quot;local pending&amp;quot; state until the TubeStateChanged signal is         emitted.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_channel_type_stream_tube_run_accept_stream_tube (TpProxy *proxy,
    gint timeout_ms,
    guint in_address_type,
    guint in_access_control,
    const GValue *in_access_control_param,
    GValue **out_address,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE;
  TpProxyPendingCall *pc;
  _emp_cli_channel_type_stream_tube_run_state_accept_stream_tube state = {
      NULL /* loop */, error,
    out_address,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "AcceptStreamTube", iface,
      _emp_cli_channel_type_stream_tube_finish_running_accept_stream_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "AcceptStreamTube",
          _emp_cli_channel_type_stream_tube_collect_callback_accept_stream_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_UINT, in_address_type,
              G_TYPE_UINT, in_access_control,
              G_TYPE_VALUE, in_access_control_param,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * emp_cli_tube_handler_callback_for_handle_tube:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a HandleTube method call
 * succeeds or fails.
 */
static void
_emp_cli_tube_handler_collect_callback_handle_tube (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_emp_cli_tube_handler_invoke_callback_handle_tube (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  emp_cli_tube_handler_callback_for_handle_tube callback = (emp_cli_tube_handler_callback_for_handle_tube) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_tube_handler_call_handle_tube:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel: Used to pass an 'in' argument: The object-path of the tubes channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @callback: called when the method call succeeds or fails;
 *   may be %NULL to make a "fire and forget" call with no 
 *   reply tracking
 * @user_data: user-supplied data passed to the callback;
 *   must be %NULL if @callback is %NULL
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled;
 *   must be %NULL if @callback is %NULL
 * @weak_object: If not %NULL, a #GObject which will be 
 *   weakly referenced; if it is destroyed, this call 
 *   will automatically be cancelled. Must be %NULL if 
 *   @callback is %NULL
 *
 * Start a HandleTube method call.
 *
 * Called when a tube handler should handle a new tube within a tubes channel.
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
emp_cli_tube_handler_call_handle_tube (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    emp_cli_tube_handler_callback_for_handle_tube callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = EMP_IFACE_QUARK_TUBE_HANDLER;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL || user_data == NULL, NULL);
  g_return_val_if_fail (callback != NULL || destroy == NULL, NULL);
  g_return_val_if_fail (callback != NULL || weak_object == NULL, NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);

      if (destroy != NULL)
        destroy (user_data);

      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "HandleTube",
          G_TYPE_STRING, in_Bus_Name,
          DBUS_TYPE_G_OBJECT_PATH, in_Connection,
          DBUS_TYPE_G_OBJECT_PATH, in_Channel,
          G_TYPE_UINT, in_Handle_Type,
          G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "HandleTube", iface,
          _emp_cli_tube_handler_invoke_callback_handle_tube,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "HandleTube",
              _emp_cli_tube_handler_collect_callback_handle_tube,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    unsigned success:1;
    unsigned completed:1;
} _emp_cli_tube_handler_run_state_handle_tube;
static void
_emp_cli_tube_handler_finish_running_handle_tube (TpProxy *self G_GNUC_UNUSED,
    GError *error,
    GValueArray *args,
    GCallback unused G_GNUC_UNUSED,
    gpointer user_data G_GNUC_UNUSED,
    GObject *unused2 G_GNUC_UNUSED)
{
  _emp_cli_tube_handler_run_state_handle_tube *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * emp_cli_tube_handler_run_handle_tube:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Bus_Name: Used to pass an 'in' argument: The bus name of the connection and channel
 * @in_Connection: Used to pass an 'in' argument: The object-path of the connection that owns the channel
 * @in_Channel: Used to pass an 'in' argument: The object-path of the tubes channel
 * @in_Handle_Type: Used to pass an 'in' argument: The type of the handle that the channel communicates           with, or 0 if there is no associated handle
 * @in_Handle: Used to pass an 'in' argument: The handle that the channel communicates with,           or 0 if there is no associated handle
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method HandleTube and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * Called when a tube handler should handle a new tube within a tubes channel.
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
emp_cli_tube_handler_run_handle_tube (TpProxy *proxy,
    gint timeout_ms,
    const gchar *in_Bus_Name,
    const gchar *in_Connection,
    const gchar *in_Channel,
    guint in_Handle_Type,
    guint in_Handle,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = EMP_IFACE_QUARK_TUBE_HANDLER;
  TpProxyPendingCall *pc;
  _emp_cli_tube_handler_run_state_handle_tube state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "HandleTube", iface,
      _emp_cli_tube_handler_finish_running_handle_tube,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "HandleTube",
          _emp_cli_tube_handler_collect_callback_handle_tube,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_STRING, in_Bus_Name,
              DBUS_TYPE_G_OBJECT_PATH, in_Connection,
              DBUS_TYPE_G_OBJECT_PATH, in_Channel,
              G_TYPE_UINT, in_Handle_Type,
              G_TYPE_UINT, in_Handle,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/*
 * emp_cli_misc_add_signals:
 * @self: the #TpProxy
 * @quark: a quark whose string value is the interface
 *   name whose signals should be added
 * @proxy: the D-Bus proxy to which to add the signals
 * @unused: not used for anything
 *
 * Tell dbus-glib that @proxy has the signatures of all
 * signals on the given interface, if it's one we
 * support.
 *
 * This function should be used as a signal handler for
 * #TpProxy::interface-added.
 */
static void
emp_cli_misc_add_signals (TpProxy *self G_GNUC_UNUSED,
    guint quark,
    DBusGProxy *proxy,
    gpointer unused G_GNUC_UNUSED)
{
  if (quark == EMP_IFACE_QUARK_CHANNEL_INTERFACE_TUBE)
    emp_cli_add_signals_for_channel_interface_tube (proxy);
  if (quark == EMP_IFACE_QUARK_CHANNEL_TYPE_DBUS_TUBE)
    emp_cli_add_signals_for_channel_type_dbus_tube (proxy);
  if (quark == EMP_IFACE_QUARK_CHANNEL_TYPE_STREAM_TUBE)
    emp_cli_add_signals_for_channel_type_stream_tube (proxy);
}
